using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using InternshipSystem.Core;

namespace InternshipSystem.Repository
{
    public class DatabaseFiller
    {
        public DatabaseFiller(InternshipDbContext context)
        {
            Context = context;
        }

        public InternshipDbContext Context { get; }

        public async Task FillCompany()
        {
            var companies = new List<Company>
            {
                new Company 
                {
                    Name = "Intel",
                    SiteAddress = new Uri("https://www.intel.com/content/www/us/en/jobs/locations/poland.html"),
                    Nip = "9570752316",
                    Range = RangeOfActivities.International,
                    Branches = new List<BranchOffice>() 
                    {
                        new BranchOffice() 
                        {
                            Address = new Address() 
                            {
                                City = "Gdańsk",
                                Street = "ul. Słowackiego",
                                Building = "173",
                                PostalCode = "80-298",
                                Country = "Poland"
                            }
                        },
                        new BranchOffice() 
                        {
                            Address = new Address() 
                            {
                                City = "Gdańsk",
                                Street = "Jana z Kolna",
                                Building = "11",
                                PostalCode = "80-001",
                                Country = "Poland"
                            }
                        },
                        new BranchOffice() 
                        {
                            Address = new Address() 
                            {
                                City = "Boston",
                                Street = "State St",
                                Building = "53",
                                PostalCode = "MA 02109",
                                Country = "Stany Zjednoczone"
                            }
                        }
                    }
                },
                new Company
                {
                    Name = "Asseco Poland",
                    SiteAddress = new Uri("http://pl.asseco.com"),
                    Nip = "5842068320",
                    Range = RangeOfActivities.National,
                    Branches = new List<BranchOffice>
                    {
                        new BranchOffice
                        {
                            Address = new Address
                            {
                                City = "Gdańsk",
                                Street = "ul. Podolska",
                                Building = "21",
                                PostalCode = "81-321",
                                Country = "Poland"
                            }
                        },
                        new BranchOffice
                        {
                            Address = new Address
                            {
                                City = "Wrocław",
                                Street = "Traugutta",
                                Building = "1/7",
                                PostalCode = "50-449",
                                Country = "Poland"
                            }
                        }
                    }
                }
            };
            Context.Companies.AddRange(companies);
            await Context.SaveChangesAsync();
        } 

        public async Task FillInterns() 
        {
            var interns = new List<Intern>
            {
                new Intern {
                    Name = "Jan",
                    Surname = "Kowalski",
                    AlbumNumber = "123456",
                    Email = "s123456@student.pg.edu.pl",
                    Course = new Course
                    {
                        Name = "Informatyka",
                        DesiredSemesters = new List<int> { 4, 6 }
                    }
                },
                new Intern {
                    Name = "Adam",
                    Surname = "Kołek",
                    AlbumNumber = "102137",
                    Email = "s102137@student.pg.edu.pl",
                    Course = new Course 
                    {
                        Name = "Robotyka",
                        DesiredSemesters = new List<int> { 6 }
                    }
                }
            };
            Context.Interns.AddRange(interns);
            await Context.SaveChangesAsync();
        }

        public async Task FillInternshipTypes() {
            var internshipTypes = new List<InternshipType>
            {
                new InternshipType 
                {
                    Type = "FreeInternship",
                    Description = "Praktyka bezpłatna",
                },
                new InternshipType 
                {
                    Type = "GraduateInternship"
                },
                new InternshipType 
                {
                    Type = "FreeApprenticeship"
                },
                new InternshipType
                {
                    Type = "PaidApprenticeship",
                    Description = "np. przemysłowy"
                },
                new InternshipType
                {
                    Type = "ForeignInternship",
                    Description = "np. IAESTE, ERASMUS"
                },
                new InternshipType
                {
                    Type = "UOP"
                },
                new InternshipType
                {
                    Type = "UD"
                },
                new InternshipType
                {
                    Type = "UZ"
                },
                new InternshipType
                {
                    Type = "Other",
                    Description = "Należy wprowadzić samodzielnie"
                }
            };
            Context.InternshipTypes.AddRange(internshipTypes);
            await Context.SaveChangesAsync();
        }

        // TODO: rewrite later
        // public async Task FillEditions() {
        //     var editions = new List<Edition>
        //     {
        //         new Edition 
        //         {
        //             StartDate = new DateTime(2000, 5, 10),
        //             EndDate = new DateTime(2000, 11, 10),
        //             IPPDeadlineDate = new DateTime(2000, 9, 30),
        //             Subjects = new List<InternshipProgramSubject>
        //             {
        //                 new InternshipProgramSubject
        //                 {
        //                     Course = new Course
        //                     {
        //                         Name = "Informatyka"
        //                     }
        //                 },
        //                 new InternshipProgramSubject
        //                 {
        //                     Course = new Course
        //                     {
        //                         Name = "Budowanie UI"
        //                     }
        //                 },
        //                 new InternshipProgramSubject
        //                 {
        //                     Course = new Course
        //                     {
        //                         Name = "Budowanie Back End"
        //                     }
        //                 }
        //             }
        //         }
        //     };
        //     Context.Editions.AddRange(editions);
        //     await Context.SaveChangesAsync();
        // }

        public async Task FillInternShips() {
            var intenrships = new List<Internship>
            {
                new Internship
                {
                    Intern = Context.Interns.Where(i => i.AlbumNumber == "123456").Single(),
                    Edition = null, //TODO: Context.Editions.Where(e => e.StartDate.Equals(new DateTime(2000, 5, 10))).Single(),
                    BranchOffice = new BranchOffice() 
                    {
                        Address = new Address() 
                        {
                            City = "Gdańsk",
                            Street = "ul. Słowackiego",
                            Building = "173",
                            PostalCode = "80-298",
                            Country = "Poland"
                        }
                    },
                    Type = Context.InternshipTypes.Where(t => t.Type == "UOP").Single(),
                    StartDate = new DateTime(2000, 7, 1),
                    EndDate = new DateTime(2000, 8, 30),
                    IsAccepted = false,
                    DeanAcceptance = new DeanAcceptance
                    {
                        AcceptanceDate = new DateTime(2000, 6, 26),
                        IsDeansAcceptanceRequired = true,
                        Reason = DeanAcceptanceReason.Semester
                    },
                    Insurance = new Insurance
                    {
                        InsuranceDeliveryDate = new DateTime(2000, 6, 29),
                        IsInsuranceRequired = true
                    },
                    InternshipLengthInWeeks = 9,
                    Grade = 0,
                    Program = new List<InternshipProgramSubject>
                    {
                        new InternshipProgramSubject
                        {
                            Course = new Course
                            {
                                Name = "Bazy Danych"   
                            }
                        },
                        new InternshipProgramSubject
                        {
                            Course = new Course
                            {
                                Name = "Design UI"   
                            }
                        }
                    },
                    Mentor = new Mentor
                    {
                        Name = "Horacy",
                        Surname = "Wościcki",
                        Email = "howos@intel.com",
                        Phone = "605-555-555"
                    }
                }
            };
            Context.Internships.AddRange(intenrships);
            await Context.SaveChangesAsync();
        }
    } 
}